<?php
/**
 *  店铺模板装修（2个APP端和商城端的代码完全一样）
 */
class YdTemplate {

    /**
     *  模板数据解析
     * @param $pageData 页面json字符串
     */
    public function parsePageData($pageData, $tabbarData=false){
        $data = json_decode($pageData ,true);
        if(empty($data)){
            //如果不存在，就返回默认数据
            $Parameters = array();
            $Parameters['PageBgColor'] = '#f5f5f5';
            $Parameters['MarginSize'] = 0;
            $Parameters['Type'] = 'head';
            $Parameters['TitleColor'] = '#000000';
            $Parameters['TitleBgColor'] = '#ffffff';
            $data['Parameters'] = $Parameters;
            $data['Components'] = array();
            return $data;
        }
        //工具栏数据
        if($tabbarData){
            $data['Tabbar'] = $tabbarData;
        }
        //对组件进行排序
        $this->_sortComponent($data['Components']);

        //设置组件默认值
        $this->_setComponentDefault($data);
        //2.解析组件参数
        foreach($data["Components"] as $k=>$v){
            $Components = &$data["Components"][$k];
            $type = $v['Type'];
            if('search' == $type){ //搜索框
                //格式： {Type:'search',   Order:50,   ShowScan:1,  SearchText:"名称 货号" }
                //不需要解析参数
            }elseif('banner' == $type){ //轮播图
                //格式：{Type:2,Order:50,DataList:[{BannerPicture:"", LinkType:1, LinkValue:"" }]}
                AddResDomain($Components['DataList'], 'Picture');
                $this->getLinkUrl($Components['DataList']);
            }elseif('announce' == $type){ //公告
                //不需要解析参数
            }elseif('class' == $type){ //分类导航：
                //格式：{"Type":"class", "Order":1,  "ClassList":[{"Name":"栏目1", "Picture":"/1.png", "LinkType":2, "LinkValue":"2" }]},
                AddResDomain($Components['DataList'], 'Picture');
                $this->getLinkUrl($Components['DataList']);
            }elseif('goods' == $type){ // 商品列表
                // 格式 { "Type":"goods", "Order":90, "Style":"1", "Title":"热卖商品", "Source":"-2", "Count":"6",
                // "ShowPrice":"1", "ShowName":"1", "ShowBuy":1, "ShowLinePrice":1},
                $DataList = $this->getGoodsData($Components['Count'], $Components['Source']);
                $Components['DataList'] = $DataList;
            }elseif('picture' == $type){ //图片列表
                AddResDomain($Components['DataList'], 'Picture');
                $this->getLinkUrl($Components['DataList']);
            }elseif('video' == $type){ //视频
                $Components['VideoUrl'] = AddResDomain($Components['VideoUrl']);
            }elseif('title' == $type){ //标题
                $Components['Picture'] = AddResDomain($Components['Picture']);
                $this->getLinkUrl($Components);
            }elseif('richtext' == $type){ //富文本
                //格式：{ "Type":"richtext", "Order":95, "Content":"您好，这是一个富文本！"}
                //不需要解析参数
                $Components['Content'] = AddContentResDomain($Components['Content']);
            }elseif('blank' == $type){ //空白
                //格式：{ "Type":"blank", "Order":96, "Height":"8px"}
                //不需要解析参数
            }elseif('line' == $type){ //辅助线
                //格式：{ "Type":"line", "Order":97, "Style":"1"}
                //不需要解析参数
            }elseif('content' == $type){ // 内容列表
                // 格式 { "Type":"content", "Order":90, "Style":"1", "Title":"热卖商品", "Source":"-2", "Count":"6",
                // "ShowPrice":"1", "ShowName":"1", "ShowBuy":1, "ShowLinePrice":1},
                $Components['DataList'] = $this->getInfoData($Components);
            }elseif('second' == $type){ // 秒杀
                $Components['DataList'] = $this->getSecondData($Components);
            }elseif('guestbook'==$type){ //留言
                $Components['DataList'] =  get_model(6);
            }elseif('feedback'==$type){ //反馈
                $Components['DataList'] =  get_model(37);
            }elseif('contact'==$type){
                $Components['Picture'] = AddResDomain($Components['Picture']);
                $Components['Logo'] = AddResDomain($Components['Logo']);
            }
        }
        return $data;
    }

    /**
     * 获取优惠券数据
     */
    function getCouponData($n=8){
        $m = D('Coupon');
        $params = array();
        $params['CouponStatus'] = 1; //正在进行中的优惠券
        $data = $m->getCoupon($params);
        $length = count($data);
        if(empty($n)) $n = 8;
        if($length>$n){
            $data = array_slice($data, 0, $n);
        }
        return $data;
    }

    /**
     * 获取商品组件数据（添加组件和编辑组件时会更新数据）
     */
    function getGoodsData($count, $source){
        $m = D('Goods');
        $params = array();
        $params['IsEnable'] = 1; //一定已启用的商品
        //Source：-1全部商品、IsNew新品、IsHot热卖、IsRecommend推荐、分类1、分类2）
        if('-1' == $source){
            //全部商品
        }elseif('IsNew' == $source){
            $params['IsNew'] = 1;
        }elseif('IsHot' == $source){
            $params['IsHot'] = 1;
        }elseif('IsRecommend' == $source){
            $params['IsRecommend'] = 1;
        }elseif($source > 0){
            $params['CategoryID'] = $source;
        }
        if(empty($count)) $count = 20;
        $data = $m->getShopGoods(0, $count, $params);
        return $data;
    }

    /**
     * 获取文字数据
     */
    function getInfoData($Components){
        $params = array();
        $params['Field'] = 'InfoID,InfoTitle,ChannelID,ChannelName,InfoTime,InfoHit,InfoPicture';
        //频道ID
        $ChannelID = $Components['Source']>0 ? $Components['Source'] : 0;
        //属性1：推荐，2热门
        $AttributeMap = array(1=>'recommend', 2=>'hot');
        $key = $Components['Attribute'];
        if(isset($AttributeMap[$key]) && $ChannelID>0){
            $LabelID = ChannelLabelID($AttributeMap[$key], $ChannelID);
        }else{
            $LabelID = '';
        }

        //排序
        $orderMap = array(1=>'InfoOrder ASC, InfoTime DESC', 2=>'InfoHit DESC', 3=>'a.InfoTime desc');
        $key = $Components['OrderBy'];
        $orderby = isset($orderMap[$key]) ? $orderMap[$key] : 1;
        $count = intval($Components['Count']);
        $m = D('Admin/Info');
        $data = $m->getInfo(0, $count, $ChannelID, 1, 1, $LabelID, '', -1, 0, 1, $orderby, -1, $params);
        AddResDomain($data, 'InfoPicture');
        FriendDate($data, 'InfoTime', true);
        return $data;
    }

    /**
     * 获取链接地址
     */
    private function getLinkUrl(&$data){
        if(is_array($data[0])){ //二维数组
            foreach($data as $k=>$v){
                $data[$k]['LinkUrl'] = $this->getLinkUrlByValue($v['LinkType'], $v['LinkValue']);
            }
        }else{
            $data['LinkUrl'] = $this->getLinkUrlByValue($data['LinkType'], $data['LinkValue']);
        }
    }

    private function getLinkUrlByValue($type,$value){
        $type = intval($type);
        $url = '';
        switch($type){
            case 0: //不跳转
                $url = '';
                break;
            case 1: //商品详情
                $url = "/pages/goodsDetail/goodsDetail?GoodsID={$value}";
                break;
            case 2: //商品分类
                $url = $this->getCategoryLinkUrl($value);
                break;
            case 3: //内容， value对应InfoID
                $url = "/pages/content/showInfo/showInfo?infoid={$value}";
                break;
            case 4: //频道，$value：对应ChannelID，-1：表示
                $ChannelModelID = ($value>0) ? ChannelModelID($value) : 0;
                if($ChannelModelID == 32){
                    $url = "/pages/content/showchannel/showchannel?channelid={$value}";
                }else{
                    $url = "/pages/content/info/info?channelid={$value}";
                }
                break;
            case 7: //拨号
                $url = $value;
                break;
            case 9: //外部链接
                $url = $value;
                break;
        }
        return $url;
    }

    /**
     * 获取商品分类链接
     */
    private function getCategoryLinkUrl($CategoryID){
        static $map = array();
        if(empty($map)){
            $data = YdCache::readDictionaryData('Category');
            foreach($data as $v){
                $key = $v['ID'];
                $map[$key] = $v['ParentID'];
            }
        }
        //1:表示tab2，1级，2级，3级
        $url = array();
        $url[] = $CategoryID;
        if($CategoryID > 0){
            while(true){
                $ParentID = isset($map[$CategoryID]) ? $map[$CategoryID]: 0;
                if($ParentID == 0) break;
                $url[] = $ParentID;
                $CategoryID = $ParentID;
            }
        }
        //分类链接格式：1（表示tab2）,1级,2级,3级,4级...
        $url[] = 1;
        $url = implode(',', array_reverse($url));
        return $url;
    }

    /**
     * 对组件数组进行排序
     * 在预览或小程序端显示时，使用Flex布局来排序，不需要在后端进行排序
     */
    private function _sortComponent(&$data){
        //$order = array_column($data,'Order');
        //array_multisort($order, SORT_ASC, SORT_NUMERIC, $data);
    }

    /**
     * 设置组件默认值
     */
    private function _setComponentDefault(&$data){
        $params = &$data['Parameters'];
        //导航标题背景
        if(empty($params['TitleBgColor'])) {
            $params['TitleBgColor'] = '#FFFFFF';
        }
        if(empty($params['PageBgColor'])) {
            $params['PageBgColor'] = '#FFFFFF';
        }
        if(empty($params['MarginSize'])) {
            $params['MarginSize'] = 0;
        }
        //获取组件默认值关联数组
        $map = array();
        $datalist = Self::getComponent();
        foreach($datalist as $v){
            $key = $v['ComponentType'];
            $map[$key] = $v['Default'];
        }
        // 设置组件默认值
        $Components= &$data['Components'];
        foreach($Components as $k=>$v){
            $key = $v['Type'];
            foreach($map[$key] as $itemName=>$defaultValue){
                if($Components[$k][$itemName]==='' || is_null($Components[$k][$itemName])){
                    $Components[$k][$itemName] = $defaultValue;
                }
            }
        }
    }

    //======================模板字典数据 开始======================
    //获取模板所有字典数据
    static public function getAllDictionary(){
        $data = array();
        $data['Component'] = YdTemplate::getComponent(); //组件
        $data['LinkType'] = YdTemplate::getLinkType(); //跳转类型
        //$data['GoodsSource'] = YdTemplate::getGoodsSource();
        //幻灯片播放时间间隔
        $data['BannerTime'] = array(
            array('Value'=>2, 'Name'=>'2秒'),
            array('Value'=>3, 'Name'=>'3秒'),
            array('Value'=>4, 'Name'=>'4秒'),
            array('Value'=>5, 'Name'=>'5秒'),
            array('Value'=>6, 'Name'=>'6秒'),
            array('Value'=>7, 'Name'=>'7秒'),
            array('Value'=>8, 'Name'=>'8秒'),
        );
        return $data;
    }

    //获取组件，在app端显示的时候，要以ComponentType作为图片名称显示图片
    static  public function getComponent(){
        //Default：表示默认值，主要用于重置
        $data = array();
        $data[] = array('ComponentGroupID'=>1, 'ComponentGroupName'=>'基础组件', 'ComponentList'=>array(
            /*====================基础组件 开始====================*/
            //自由画板
            array('Type'=>'container', 'Name'=>'自由容器', 'Default'=>array(
                'BgColor'=>'#FFF', 'BgPicture'=>'', 'Height'=>'300'
            ),  'Styles'=>''),
            //搜索框
            array('Type'=>'search', 'Name'=>'搜索框',
                'Default'=>array(
                    'Style'=>1, 'SearchText'=>'名称 货号 条码',  'SearchBgColor'=>'#F0F0F0',
                    'Color'=>'#000', 'BgColor'=>'#FFF', 'ShowScan'=>1, 'ScanColor'=>'#FF0033'
                ),  'Styles'=>array(
                array('StyleID'=>1, 'StyleName'=>'圆角'),
                array('StyleID'=>2, 'StyleName'=>'直角'),
            )),
            //轮播图
            array('Type'=>'banner', 'Name'=>'轮播图', 'Default'=>'',  'Styles'=>''),
            //公告
            array('Type'=>'announce', 'Name'=>'公告',
                'Default'=>array('Color'=>'#333', 'BgColor'=>'#fff', 'Margin'=>0), 'Styles'=>''),
            //分类导航
            array('Type'=>'class', 'Name'=>'分类导航', 'Default'=>'', 'Styles'=>''),
            //商品
            /*
            array('Type'=>'goods', 'Name'=>'商品', 'Default'=>'', 'Styles'=>array(
                array('StyleID'=>1, 'StyleName'=>'一行二个'),
                array('StyleID'=>2, 'StyleName'=>'一行三个'),
                array('StyleID'=>3, 'StyleName'=>'详细列表'),
                array('StyleID'=>4, 'StyleName'=>'大图模式'),
                array('StyleID'=>5, 'StyleName'=>'横向滑动'),
            )),
            */
            //内容
            array('Type'=>'content', 'Name'=>'内容列表', 'Default'=>'', 'Styles'=>array(
                array('StyleID'=>1, 'StyleName'=>'左文右图'),
                array('StyleID'=>2, 'StyleName'=>'左图右文'),
                array('StyleID'=>3, 'StyleName'=>'大图模式'),
                array('StyleID'=>4, 'StyleName'=>'一行二个'),
                array('StyleID'=>5, 'StyleName'=>'文字列表'),
                array('StyleID'=>6, 'StyleName'=>'横向滑动'),
            )),
            //图片（不能在这里设置默认参数，使用APP的默认参数，底部有添加图片的按钮）
            array('Type'=>'picture', 'Name'=>'图片', 'Default'=>'', 'Styles'=>array(
                array('StyleID'=>1, 'StyleName'=>'一行一张'),
                array('StyleID'=>2, 'StyleName'=>'一行二张'),

                array('StyleID'=>3, 'StyleName'=>'一行三张'),
                array('StyleID'=>31, 'StyleName'=>'三张左大'),
                array('StyleID'=>32, 'StyleName'=>'三张右大'),

                array('StyleID'=>4, 'StyleName'=>'一行四张'),
                array('StyleID'=>41, 'StyleName'=>'二左二右'),

                array('StyleID'=>9, 'StyleName'=>'横向滑动'),
            )),
            //视频
            array('Type'=>'video', 'Name'=>'视频', 'Default'=>'', 'Styles'=>''),
            //标题
            array('Type'=>'title', 'Name'=>'标题栏', 'Default'=>array(
                'Color'=>'#333', 'FontSize'=>'20', 'BgColor'=>'#FFF',
                'ShowLine'=>1, 'LineColor'=>'#FF0033', 'ArrowColor'=>'#CCC'
            ), 'Styles'=>array(
                array('StyleID'=>1, 'StyleName'=>'文字标题'),
                array('StyleID'=>2, 'StyleName'=>'文字+更多'),
                array('StyleID'=>3, 'StyleName'=>'文字+箭头'),
                array('StyleID'=>4, 'StyleName'=>'图片标题'),
            )),
            //辅助线
            array('Type'=>'line', 'Name'=>'辅助线', 'Default'=>'','Styles'=>array(
                array('StyleID'=>1, 'StyleName'=>'实线'),
                array('StyleID'=>2, 'StyleName'=>'虚线'),
                array('StyleID'=>3, 'StyleName'=>'点线'),
            )),
            //空白占位符
            array('Type'=>'blank', 'Name'=>'空白占位符', 'Default'=>'','Styles'=>''),
            //富文本
            array('Type'=>'richtext', 'Name'=>'文本', 'Default'=>'','Styles'=>''),
            /*====================基础组件 结束====================*/
        ));
        $data[] = array('ComponentGroupID'=>2, 'ComponentGroupName'=>'互动组件', 'ComponentList'=>array(
            /*====================营销组件 开始====================*/
            //浮动按钮
            array('Type'=>'fab', 'Name'=>'浮动按钮', 'Default'=>array(), 'Styles'=>array()),
            //留言
            array('Type'=>'guestbook', 'Name'=>'留言', 'Default'=>array(), 'Styles'=>array()),
            //反馈
            array('Type'=>'feedback', 'Name'=>'反馈', 'Default'=>array(), 'Styles'=>array()),
            /*====================营销组件 结束====================*/
        ));
        return $data;
    }

    //获取跳转链接类型
    static  public function getLinkType(){
        $data = array(
            array('LinkTypeID'=>0, 'LinkTypeName'=>'不跳转'),
            //array('LinkTypeID'=>1, 'LinkTypeName'=>'商品'),
            //array('LinkTypeID'=>2, 'LinkTypeName'=>'商品分类'),
            //文章和文章分类
            array('LinkTypeID'=>3, 'LinkTypeName'=>'文章'),
            array('LinkTypeID'=>4, 'LinkTypeName'=>'频道'),
            array('LinkTypeID'=>5, 'LinkTypeName'=>'页面'),

            array('LinkTypeID'=>7, 'LinkTypeName'=>'拨号'),
            array('LinkTypeID'=>9, 'LinkTypeName'=>'外部链接'),
        );
        return $data;
    }

    //获取商品数据源
    static public function getGoodsSource(){
        //0全部商品、-1新品、-2热卖、-3推荐、分类1、分类2）
        $allData = array(
            array('CategoryID'=>'-1', 'CategoryName'=>'全部商品', 'Depth'=>1),
            array('CategoryID'=>'IsNew', 'CategoryName'=>'新品', 'Depth'=>1),
            array('CategoryID'=>'IsHot', 'CategoryName'=>'热卖', 'Depth'=>1),
            array('CategoryID'=>'IsRecommend', 'CategoryName'=>'推荐', 'Depth'=>1),
        );
        $data = YdCache::readDictionaryData('Category');
        foreach($data as $v){
            $name = $v['Name'];
            $depth = $v['Depth'];
            if($depth > 1){
                //在app端，需要使用innerHtml显示
                $space = str_repeat('&nbsp;&nbsp;', $depth);
                $name = "{$space}|--{$name}";
            }
            $allData[] = array('CategoryID'=>$v['ID'], 'CategoryName'=>$name, 'Depth'=>$depth);
        }
        return $allData;
    }

    /**
     * 获取频道来源
     */
    static public function getChannelSource($params){
        $m = D('Admin/Channel');
        $data = $m->getChannelSource(0, -1, '');
        if( empty($data) ) return false;  //当$data为false时，count($data)返回1，因此必选先判断$data是否为空
        //找出ChannelDepth最大值
        $maxDepth = -9999;
        foreach($data as $k=>$v){
            if( $v['ChannelDepth'] > $maxDepth ) {
                $maxDepth = $v['ChannelDepth'];
            }
        }
        foreach($data as $k=>$v){
            $data[$k]['ChannelDepth'] = ($maxDepth - $data[$k]['ChannelDepth'] + 1);
        }
        if(isset($params['channel1'])){
            $first = array('ChannelID'=>-1, 'ChannelName'=>'全部频道', 'ParentID'=>'0', 'ChannelDepth'=>1,'ChannelModelID'=>0);
            array_unshift($data, $first);
        }
        return  $data;
    }
    //======================模板字典数据 结束======================
}