<?php
/**
 * Youdian Content Management System
 * Copyright (C) YoudianSoft Co.,Ltd (http://www.youdiancms.com). All rights reserved. 
 */
class YdTemplateConfig{
	private $_fileName = null;
	private $_languageMark = null;
	private $_order = null;  //排序数组
    private $_lastError = ''; //最后一次错误
	function __construct($fileName, $languageMark){
		$this->_fileName = $fileName;
		$this->_languageMark = $languageMark;
		unset($this->_order);
	}

	function getLastError(){
	    return $this->_lastError;
    }
	
	//获取分组
	function getGroup(){
		if( !file_exists($this->_fileName) ) return false;
		$xml = @simplexml_load_file( $this->_fileName );
		if(empty($xml)) return false;
		$obj = $xml->groups->group;
		$n = count($obj);
		$group = array();
		for($i=0; $i < $n; $i++){
			$group[] = array('DisplayName' => (string)$obj[$i]['name'],'AttributeID' => (string)$obj[$i]['id']);
		}
		if(empty($group)) return false;
		return $group;
	}
	
	//获取模板配置属性
	function getAttribute($params=array()){
		if( !file_exists($this->_fileName) ) return false;
		$xml = @simplexml_load_file( $this->_fileName );
		if(empty($xml)) return false;
		unset($this->_order); //清空排序数组
		$lang = $this->_languageMark;
        $a1 = array();
		if(!empty($xml->global->var)){
            $a1 = $this->_getVar($xml->global->var);
        }
		if(empty($a1)) $a1 = array();

		$a2 = $this->_getVar($xml->$lang->var, $params);
		if(empty($a2)) $a2 = array();
		$attribute = array_merge($a1, $a2);
		//按order进行排序
		array_multisort ($this->_order, SORT_ASC, SORT_NUMERIC, $attribute);
		return $attribute;
	}
	
	//常见模板配置文件
	function create(){
		
	}
	
	//保存配置
	function save($data){
		unset($data['__hash__']);
		if(empty($data)) return false;
		if( !file_exists($this->_fileName) ) {
		    $result = false;
        }
		
		//预处理，对checkbox转化为字符串
		foreach ($data as $k=>$v){
			if( is_array($v) ){
				$data[$k] = implode(',', $v);
			}
		}
		$xml = @simplexml_load_file( $this->_fileName );
		if(empty($xml)) return false;
		//保存全局参数========================
		$a1 = &$xml->global->var;
		$n = count($a1);
		for($i=0; $i < $n; $i++){
			$key = (string)$a1[$i]['name'];
			if( key_exists($key, $data)){
				$a1[$i]['value'] = $data[$key];
			}
		}
		//==================================
		
		//保存当前语言配置参数====================
		$lang = $this->_languageMark;
		$a2 = &$xml->$lang->var;
		$n = count($a2);
		for($i=0; $i < $n; $i++){
			$key = (string)$a2[$i]['name'];
			if( key_exists($key, $data)){
                $type = (string)$a2[$i]['type'];
                if($type =='font'){ //字体单独处理
                    $a2[$i]['value'] = $this->_getFontValue($data, $key);
                }elseif($type=='xy'){
                    $a2[$i]['value'] = $this->_getXY($data, $key);
                }elseif($type=='bg'){
                    $a2[$i]['value'] = $this->_getBg($data, $key);
                }elseif($type=='button'){
                    $a2[$i]['value'] = $this->_getButton($data, $key);
                }else{
                    $a2[$i]['value'] = $data[$key];
                }
			}
		}
		//=================================
		$result = $xml->asXML($this->_fileName);
		return $result;	
	}

    /**
     * 获取背景
     */
    private function _getBg(&$data, $name){
        //格式：0背景类型, 1背景颜色(开始),2背景颜色(结束)，3背景图片，4平铺,5背景大小自适应，6背景位置
        $keyWay = "{$name}Way";  //0
        $keyStartColor = "{$name}StartColor";
        $keyEndColor = "{$name}EndColor";  //2
        $keyImage = "{$name}Image";
        $keyRepeat = "{$name}Repeat"; //4
        $keySize = "{$name}Size";  //5
        $keyPosition = "{$name}Position"; //6
        $keyAttachment = "{$name}Attachment"; //7
        $keyAngle = "{$name}Angle"; //8

        $way = isset($data[$keyWay]) ? $data[$keyWay] : '';
        $startColor = isset($data[$keyStartColor]) ? $data[$keyStartColor] : '';
        $endColor = isset($data[$keyEndColor]) ? $data[$keyEndColor] : '';

        $image = isset($data[$keyImage]) ? $data[$keyImage] : '';
        $repeat = isset($data[$keyRepeat]) ? $data[$keyRepeat] : '';
        $size = isset($data[$keySize]) ? $data[$keySize] : '';
        $position = isset($data[$keyPosition]) ? $data[$keyPosition] : '';
        $attachment = isset($data[$keyAttachment]) ? $data[$keyAttachment] : '';
        $angle = isset($data[$keyAngle]) ? $data[$keyAngle] : '';
        $result = "{$way},{$startColor},{$endColor},{$image},{$repeat},{$size},{$position},{$attachment},{$angle}";
        return $result;
    }

    /**
     * 获取XY坐标
     */
    private function _getXy(&$data, $name){
        $keyX = "{$name}X";
        $keyY = "{$name}Y";
        $x = isset($data[$keyX]) ? $data[$keyX] : '';
        $y = isset($data[$keyY]) ? $data[$keyY] : '';
        $result = "left:{$x}px;top:{$y}px;";
        return $result;
    }

    /**
     * 获取字体的值
     */
	private function _getFontValue(&$data, $name){
        $value = array();
	    $list = array('Family', 'Size', 'Color', 'LineHeight' , 'Bold', 'Italic', 'Underline', 'Line');
	    foreach($list as $v){
	        $key = $name.$v;
	        $temp = isset($data[$key]) ? $data[$key] : '';
	        $value[] = $temp;
        }
	    $value = implode(',', $value);
	    return $value;
    }

    /**
     * 获取按钮
     */
    private function _getButton(&$data, $name){
        //格式：0名称、1链接、2宽度、3圆角、4边框、5边框颜色、6:悬浮颜色
        $btnName = $data["{$name}Name"];
        $link = $data["{$name}Link"];
        $width = $data["{$name}Width"];
        $radius = $data["{$name}Radius"];
        $border = $data["{$name}Border"];
        $borderColor = $data["{$name}Color"];
        $hoverColor = $data["{$name}HoverColor"];
        $result = "{$btnName},{$link},{$width},{$radius},{$border},{$borderColor},{$hoverColor}";
        return $result;
    }
	
	//获取配置数据
	function getData(){
		$data = false;
		$xml = @simplexml_load_file( $this->_fileName );
		if(empty($xml)) return false;
		//获取全局参数
		$a1 = &$xml->global->var;
		$n = count($a1);
		for($i=0; $i < $n; $i++){
			$key = (string)$a1[$i]['name'];
			$data[$key] = (string)$a1[$i]['value'];
		}
		
		//获取语言参数
		$lang = $this->_languageMark;
		$a2 = &$xml->$lang->var;
		$n = count($a2);
		for($i=0; $i < $n; $i++){
			$key = (string)$a2[$i]['name'];
			$data[$key] = (string)$a2[$i]['value'];
		}
		return $data;
	}
	
	//判断属性类型是否是可选
	function isSelected($type){
		if( stripos($type, 'checkbox') === false &&
				stripos($type, 'radio' ) === false &&
				stripos($type, 'select' )  === false  ){
			return false;
		}else{
			return true;
		}
	}

	
    private function _getVar($obj, $params=array()){
    	if(empty( $obj )) return false;
        $GroupID = isset($params['GroupID']) ? $params['GroupID'] : -1;
        $ShowVarNameInHelp = isset($params['ShowVarNameInHelp']) ? $params['ShowVarNameInHelp'] : true;
    	$Attribute = array();
    	$x=0;
    	$n = count( $obj );
    	for($i=0; $i < $n; $i++){
                if($GroupID==-1 || $GroupID == $obj[$i]['groupid']) {
                    $this->_order[] = isset($obj[$i]['order']) ? (int)$obj[$i]['order'] : 100;
                    $DisplayValue = (string)$obj[$i]['value'];
                    $DisplayType = $this->_getDefault('type', '', (string)$obj[$i]['type']);
                    $DisplayWidth = $this->_getDefault('width', $DisplayType, (string)$obj[$i]['width']);
                    $DisplayHeight = $this->_getDefault('height', $DisplayType, (string)$obj[$i]['height']);
                    $DisplayClass = $this->_getDefault('class', $DisplayType, (string)$obj[$i]['class']);
                    $DisplayHelpText = (string)$obj[$i]['help'];
                    if($ShowVarNameInHelp){
                        $DisplayHelpText .=' 配置变量名称：'.(string)$obj[$i]['name'].'';
                    }
                    $Attribute[$x] = array(
                        'FieldName'=>(string)$obj[$i]['name'],
                        'DisplayName'=>(string)$obj[$i]['title'],
                        'DisplayHelpText'=>$DisplayHelpText,
                        'DisplayType'=>$DisplayType,
                        'DisplayWidth'=>$DisplayWidth,
                        'DisplayHeight'=>$DisplayHeight,
                        'DisplayClass'=>$DisplayClass,
                        'GroupID'=>(string)$obj[$i]['groupid'],
                        'Parameter'=>(string)$obj[$i]['parameter'],
                        'Tab'=>(string)$obj[$i]['tab']
                    );
                    $Attribute[$x]['AdminGroupID'] = 1; //模板配置允许获取所有频道
                    if ( $DisplayType == 'specialselect' || $DisplayType == 'specialselectno'){
                        $Attribute[$x]['SelectedValue'] = explode(',' , $DisplayValue); //获取频道设置值
                    }else if($DisplayType == 'channelexselect' || $DisplayType == 'channelexselectno'){ //支持多选
                        $Attribute[$x]['SelectedValue'] = explode(',' ,$DisplayValue); //获取频道设置值
                    }else if( $this->isSelected( $DisplayType ) ){
                        $Attribute[$x]['SelectedValue'] = $DisplayValue;
                        $Attribute[$x]['DisplayValue'] = str_replace('@@@', "\n", (string)$obj[$i]['parameter']);
                    }else{
                        $Attribute[$x]['DisplayValue'] = $DisplayValue;
                    }
                    $x++;
                }
    	}
    	import("@.Common.YdParseModel");
    	$Attribute = parsemodel($Attribute);  //解析属性信息
    	return $Attribute;
    }
	
	//获取默认值
	private function _getDefault($key, $type, $value){
		if( !empty($value) ) {
			$data = $value;
		}else{
			$key = strtolower($key);
			switch ($key){
				case 'width': //宽度
					$dafault = array('text'=>'270px','textarea'=>'100%', 'image'=>'100%');
					$data = key_exists($type, $dafault) ? $dafault[$type] : false;
					break;
				case 'height': //高度
					$dafault = array('textarea'=>'100px');
					$data = key_exists($type, $dafault) ? $dafault[$type] : false;
					break;
				case 'class': //样式类
					$dafault = array('text'=>'textinput','textarea'=>'textinput', 'image'=>'textinput');
					$data = key_exists($type, $dafault) ? $dafault[$type] : false;
					break;
				case 'type': //类型
					$data = 'channelselect';
					break;
				default:
					$data = false;
			}
		}
		return $data;
	}

    /**
     * 删除一个分组及其，所有的变量
     */
	public function deleteGroup($GroupID){
        $fileName = $this->_fileName;
        if( !file_exists($fileName) ) {
            $this->_lastError = "模板文件不存在";
            return false;
        }
        $isWrite = yd_is_writable($fileName);
        if(!$isWrite){
            $this->_lastError = "'模板文件只读，没有权限修改！";
        }
        $xml = @simplexml_load_file( $fileName );
        if(empty($xml)) {
            $this->_lastError = "创建XML对象失败";
            return false;
        }
        //1、先删除分组==========================================
        $groups = &$xml->groups->group;
        for($i=0; $i<count($groups);$i++){
            $attributes = $groups[$i]->attributes();
            $id =  (string)$attributes['id']; //必须转化为字符串
            if($GroupID == $id){
                unset($groups[$i]);
                $i--; //删除后元素必须调整
            }
        }
        //====================================================
        //2、删除中文包变量=========================================
        $vars = &$xml->cn->var;
        $n = count($vars);
        for($i=0; $i<count($vars); $i++){
            $attributes = $vars[$i]->attributes();
            $id =  (string)$attributes['groupid']; //必须转化为字符串
            if($GroupID == $id){
                unset($vars[$i]);
                $i--; //删除后元素必须调整
            }
        }
        //=================================================
        //3、删除英文包变量======================================
        $vars = &$xml->en->var;
        for($i=0; $i<count($vars); $i++){
            $attributes = $vars[$i]->attributes();
            $id =  (string)$attributes['groupid']; //必须转化为字符串
            if($GroupID == $id){
                unset($vars[$i]);
                $i--; //删除后元素必须调整
            }
        }
        //===================================================
        $result = $xml->asXML($fileName);
        return $result;
    }

    /**
     * 获取模板最大分组ID
     */
    public function getMaxGroupID(){
        $xml = @simplexml_load_file($this->_fileName);
        if(empty($xml)) return false;
        $obj = $xml->groups->group;
        $n = count($obj);
        $max = 1;
        for($i=0; $i < $n; $i++){
            //<group id="1" name="基本设置" order="1"/>
            $id = (int)$obj[$i]['id'];
            if($id > $max){
                $max = $id;
            }
        }
        $max++; //最大值加1
        //添加的组件最小值为5000，防止与内置组件冲突
        if($max<5000) $max=5000;
        return $max;
    }

    /**
     * 添加一个配置
     * $datalist：配置项
     */
    public function addGroup($GroupID, $GroupName, $datalist){
        if(empty($datalist)){
            $this->_lastError = "分组配置为空";
            return false;
        }
        $xml = new DOMDocument();//创建DOM对象
        if(empty($xml)) return false;
        $xml->load($this->_fileName);//加载xml文件
        $groups = $xml->getElementsByTagName('groups');
        if(empty($groups)){
            $this->_lastError = "XML格式错误";
            return false;
        }
        $node = $xml->createElement('group');//创建d元素节点
        $node->setAttribute("id", $GroupID);
        $node->setAttribute("name", $GroupName);
        $node->setAttribute("order", $GroupID);
        $result = $groups[0]->appendChild($node);
        if($result){
            foreach($datalist as $v){
                $result = $this->_addVarNode($GroupID, $v, $xml);
                if(!$result){
                    $this->_lastError = "创建配置参数失败";
                    return false;
                }
            }
        }
        $result = $xml->save($this->_fileName);
        return $result;
    }

    /**
     * 添加变量节点
     */
    private function _addVarNode($groupid, $data, &$xml){
        $list = array('cn', 'en');
        foreach($list as $mark){
            //<var title="显示条数" name="TVideo2Count602" value="3"  parameter="" type="text" width="80px" help="" groupid="602" order="5"/>
            $name = str_replace($data['groupid'], $groupid, $data['name']);
            $node = $xml->createElement('var');//创建d元素节点
            $node->setAttribute("title", $data['title']);
            $node->setAttribute("name", $name);
            $node->setAttribute("type", $data['type']);
            $node->setAttribute("value", $data['value']);
            $node->setAttribute("parameter", $data['parameter']);
            if(!empty($data['tab'])){
                $node->setAttribute("tab", $data['tab']); //tab定义
            }
            $node->setAttribute("width", $data['width']);
            $node->setAttribute("help", $data['help']);
            $node->setAttribute("groupid", $groupid);
            $node->setAttribute("order", $data['order']);

            $var = $xml->getElementsByTagName($mark);
            if(empty($var)) return false;
            $result = $var[0]->appendChild($node);
            if(!$result) return false;
        }
        return true;
    }
}