<?php
//模板页面
class TemplatePageModel extends Model{
    /**
     * 获取指定模板的所有页面
     */
    function getTemplatePage($p=array()){
        $TemplateID = intval($p['TemplateID']);
        $where = "TemplateID={$TemplateID}";
        if(isset($p['Field'])){
            $field = $p['Field'];
        }else{
            $field = "TemplatePageID,TemplatePageName,TemplatePagePicture,TemplatePageType";
        }
        //按类型升序，保证首页在最前面
        $this->order("TemplatePageType ASC");
        $data = $this->where($where)->field($field)->select();
        AddResDomain($data, 'TemplatePagePicture');
        return $data;
    }

    /**
     * 查看模板页面数据
     */
    function findTemplatePage($TemplatePageID, $p = array() ){
        $where['TemplatePageID'] = intval($TemplatePageID);
        $result = $this->where($where)->find();
        if(!empty($result)){
            $type = $result['TemplatePageType'];
            //允许装修类型类别，1首页，2分类页，3列表页，4内容页，5联系我们，10会员中心是不允许装修的，组件列表页必须显示空
            $map = array(99=>'1', 1=>'1');
            $result['CanAddComponent'] = isset($map[$type]) ? 1 : 0;
            //表示获取模板相关数据
            if(isset($p['HasTemplateData']) && $p['HasTemplateData']==1){
                if(3==CLIENT_TYPE && $type != 1){ //模板装修都需要返回Tabbar
                    $tabbar = $this->getTabbar($result['TemplateID']);
                }else{ //小程序端从不返回Tabbar（在配置里返回）
                    $tabbar = false;
                }
                //APP端模板编辑的永远是草稿数据
                import('@.Common.YdTemplate');
                $t = new YdTemplate();
                $result['TemplatePageData'] = $t->parsePageData($result['TemplatePageContentDraft'], $tabbar);
                $this->_setSystemComponent($result['TemplatePageData'], $result['TemplatePageType']);
                unset($result['TemplatePageContentDraft']);
            }
        }
        return $result;
    }

    /**
     *  设置系统组件，系统组件必须返回一个值
     */
    private function _setSystemComponent(&$PageData, $PageType){
        if(!isset($PageData['Parameters'])){
            $PageData['Parameters'] = array();
        }
        $map = array(2=>'category', 3=>'list', 4=>'detail', 5=>'contact', 10=>'member');
        if(!isset($PageData['Components'])){
            if(isset($map[$PageType])){
                $PageData['Components'] = array(
                    array('Type'=>$map[$PageType])
                );
            }else{
                $PageData['Components'] = array();
            }
        }
    }

    /**
     * 删除模板页面
     */
    function deleteTemplatePage($TemplatePageID){
        if( !is_numeric($TemplatePageID) ) return false;
        $where = "TemplatePageID={$TemplatePageID}";
        $result = $this->where($where)->delete();
        return $result;
    }

    /**
     * 保存店铺模板装修
     * IsDraft：是否是草稿
     */
    function saveTemplatePage($TemplatePageID, $data){
        $TemplatePageID = intval($TemplatePageID);
        $dataToUpdate = array();
        //保存内容
        if(isset($data['TemplatePageContent'])){
            $field = 'TemplatePageContent';
            //替换其中的图片地址
            $webUrl = get_current_url();
            $TemplatePageContent = str_ireplace($webUrl, '', $data['TemplatePageContent']);
            $dataToUpdate[$field.'Draft'] = $TemplatePageContent; //保存草稿
            if(0 == $data['IsDraft']){ //保存并发布
                $dataToUpdate[$field] = $TemplatePageContent;
            }
            $this->_saveTabbar($data);
        }
        //保存页面名称
        if(isset($data['TemplatePageName'])){
            $dataToUpdate['TemplatePageName'] = $data['TemplatePageName'];
        }
        $where ="TemplatePageID={$TemplatePageID}";
        $result  = $this->where($where)->setField($dataToUpdate);
        return $result;
    }

    /**
     * 保存Tabbar的值
     */
    private function _saveTabbar(&$data){
        if(1==$data['TemplatePageType']) return; //如果是首页立即返回
        $pageData = json_decode($data['TemplatePageContent'], true);
        if(!isset($pageData['Tabbar'])) return;
        $where = "TemplateID={$data['TemplateID']} AND TemplatePageType=1";
        $contentField = (1 == $data['IsDraft']) ? 'TemplatePageContentDraft' : 'TemplatePageContent';
        $this->field("{$contentField},TemplatePageType");
        $result = $this->where($where)->find();

        //更新首页Tabbar，只有首页才有tabbar数据
        $indexPageData = json_decode($result[$contentField], true);
        $indexPageData['Tabbar'] = $pageData['Tabbar'];
        //重新保存首页
        $indexPageContent = json_encode($indexPageData);
        $dataToUpdate = array();
        $field = 'TemplatePageContent';
        $dataToUpdate[$field.'Draft'] = $indexPageContent; //保存草稿
        if(0 == $data['IsDraft']){ //保存并发布
            $dataToUpdate[$field] = $indexPageContent;
        }
        $result  = $this->where($where)->setField($dataToUpdate);
        //删除原来的tabbar数据
        unset($pageData['Tabbar']);
        $data['TemplatePageContent'] =  json_encode($pageData);
        return $result;
    }

    /**
     * 返回tabbar数据
     * $isDraft：是否获取草稿状态的tabbar
     */
    function getTabbar($TemplateID=false, $returnString=false, $isDraft=true){
        if(!$TemplateID){ //获取默认模板
            $m = D('Admin/Template');
            $TemplateID = $m->getDefaultTemplateID();
        }
        $where = "TemplateID={$TemplateID} AND TemplatePageType=1";
        $contentField = $isDraft ? 'TemplatePageContentDraft' : 'TemplatePageContent';
        $content = $this->where($where)->getField($contentField);
        $data = json_decode($content, true);
        if($returnString){
            return json_encode($data['Tabbar']);
        }else{
            return $data['Tabbar'];
        }
    }

    /**
     * 添加自定义默认页面
     */
    function addTemplatePage($data){
        $dataToAdd = array();
        $dataToAdd['TemplateID']= intval($data['TemplateID']);
        $dataToAdd['TemplatePageName'] = $data['TemplatePageName'];
        $dataToAdd['TemplatePageType'] = 99;  //99表示自定义页面
        $result  = $this->add($dataToAdd);
        return $result;
    }

    /**
     *  给模板添加默认系统页面（如：首页、联系我们、详情页面等）
     * @param $TemplateID
     */
    function addDefaultTemplatePage($TemplateID){
        $pagelist = array();
        $pagelist[] = array(
            'TemplateID'=>$TemplateID, 'TemplatePageName'=>'首页', 'TemplatePageType'=>1,
            'TemplatePageContentDraft'=>''
        );
        $pagelist[] = array(
            'TemplateID'=>$TemplateID, 'TemplatePageName'=>'分类页', 'TemplatePageType'=>2,
            'TemplatePageContentDraft'=>''
        );
        $pagelist[] = array(
            'TemplateID'=>$TemplateID, 'TemplatePageName'=>'列表页', 'TemplatePageType'=>3,
            'TemplatePageContentDraft'=>''
        );
        $pagelist[] = array(
            'TemplateID'=>$TemplateID, 'TemplatePageName'=>'详情页', 'TemplatePageType'=>4,
            'TemplatePageContentDraft'=>''
        );
        //联系我们页面=============================================================
        $pageContent = array('Components'=>array(
            array('Type'=>'contact', 'Order'=>0),
        ));
        $pagelist[] = array(
            'TemplateID'=>$TemplateID, 'TemplatePageName'=>'联系我们', 'TemplatePageType'=>5,
            'TemplatePageContentDraft'=>json_encode($pageContent)
        );
        //=====================================================================
        $result = $this->addAll($pagelist);
        return $result;
    }

    /**
     * 获取模板页面内容
     */
    function findTemplatePageContent($TemplatePageID){
        $TemplatePageID = intval($TemplatePageID);
        $where ="TemplatePageID={$TemplatePageID}";
        $result = $this->where($where)->getField('TemplatePageContent');
        return $result;
    }
}
